<?php

/**
 * Created by PhpStorm.
 * User: Administrator
 * Date: 5/7/2020
 * Time: 2:25 PM
 */

namespace Drupal\tmgmt_apostrophgroup_provider\Plugin\tmgmt\Translator;

use Drupal\Component\Utility\Xss;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\Entity\JobItem;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\Translator\TranslatableResult;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt\TranslatorPluginBase;
use Drupal\tmgmt_apostrophgroup_provider\Util\GeneralHelper;
use ZipArchive;
use Drupal\tmgmt_apostrophgroup_provider\Restclient\Api;
use Drupal\tmgmt_apostrophgroup_provider\Restclient\Model;
use Drupal\tmgmt_apostrophgroup_provider\Restclient;
use \SplFileObject;
use Drupal\tmgmt_apostrophgroup_provider\Util\AXliff;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileSystem;
use Drupal\Core\Render\Markup;

/**
 * Apostroph Translator.
 *
 * @TranslatorPlugin(
 *   id = "tmgmt_apostrophgroup_provider",
 *   label = @Translation("Apostroph Group Connector"),
 *   logo = "icons/apostroph-logo.svg",
 *   description = @Translation("Provider to send content to Apostroph Group"),
 *   ui = "Drupal\tmgmt_apostrophgroup_provider\ApostrophTranslatorUI"
 * )
 */
class ApostrophTranslator extends TranslatorPluginBase
{

  /**
   * {@inheritdoc}
   */
  public function checkTranslatable(TranslatorInterface $translator, JobInterface $job)
  {
    // Anything can be exported.
    return TranslatableResult::yes();
  }

  /**
   * Submits the translation request and sends it to the translation provider.
   *
   * During the translation process, Job::getItems() will only return job items
   * that are not already fully translated.
   *
   * @param \Drupal\tmgmt\JobInterface $job
   *   The job that should be submitted.
   *
   * @see hook_tmgmt_job_before_request_translation()
   * @see hook_tmgmt_job_after_request_translation()
   *
   * @ingroup tmgmt_remote_languages_mapping
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function requestTranslation(JobInterface $job)
  {
    // TODO: Implement requestTranslation() method.
    try {
      // Message which will be displayed using drupa_set_message to disaplay download.
      $messageTopass = 'Exported files can be downloaded here: <br/>';
      $translator = $job->getTranslator();
      $apostrophsettings = $translator->getSetting("apostroph-settings");
      $oneexportfile = $translator->getSetting("one_export_file");
      // Export files: gernerate paths and other variables.
      $exporter = new AXliff(array(), "xlf", NULL);
      $filesystem = \Drupal::service('file_system');
      $joblabel = GeneralHelper::getJobLabelNoSpeChars($job);
      $dirnameallfiles = $joblabel . '_' . $job->id() . "_" . $job->getRemoteSourceLanguage() . "_" . $job->getRemoteTargetLanguage();
      $zipName = 'zip_job_' . $dirnameallfiles . '.zip';
      $allfilespath = $job->getSetting('scheme') . '://tmgmt_apostrophgroup/ApostrophSentFiles/' . $dirnameallfiles;
      $zipPath = $allfilespath . "/" . $zipName;
      $filearrayexportedfiles = array();
      $transferfiles = array();
      // Create folder where all exported files will be stored.
      if (\Drupal::service('file_system')->prepareDirectory($allfilespath, FileSystemInterface::CREATE_DIRECTORY)) {
        // Export each item of the job in same file.
        if ($oneexportfile) {
          $labelname = $joblabel;
          $name = $labelname . "_" . $job->id() . "_all_" . $job->getRemoteSourceLanguage() . '_' . $job->getRemoteTargetLanguage() . '.xlf';
          $jobpath = $allfilespath . "/" . $name;
          $file = file_save_data($exporter->export($job), $jobpath, FileSystemInterface::EXISTS_REPLACE);
          $filearrayexportedfiles['all'] = $file;
        } else {
          // Export each item of the job in separate file.
          foreach ($job->getItems() as $item) {
            $labelname = GeneralHelper::getStringNoSpeChars($item->label());
            $name = $labelname . "_" . $job->id() . "_" . $item->id() . "_" . $job->getRemoteSourceLanguage() . '_' . $job->getRemoteTargetLanguage() . '.xlf';
            $itempath = $allfilespath . "/" . $name;
            $file = file_save_data($exporter->exportItem($item), $itempath, FileSystemInterface::EXISTS_REPLACE);
            $filearrayexportedfiles[$item->id()] = $file;
          }
        }
        // Zip the exported files.
        $ziparchive = new ZipArchive();
        $openresult = $ziparchive->open($filesystem->realpath($zipPath), ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE);
        $zipcloseresult = FALSE;
        if ($openresult) {
          foreach ($filearrayexportedfiles as $tempfile) {
            $ziparchive->addFile($filesystem->realpath($tempfile->getFileUri()), $tempfile->getFilename());
          }
          $zipcloseresult = $ziparchive->close();
          if ($zipcloseresult) {
            $zipfileobj = GeneralHelper::createFileObject($zipPath);
            \Drupal::service('file.usage')->add($zipfileobj, 'tmgmt_apostrophgroup_provider', 'tmgmt_job', $job->id());
          }
          // TODO: send zip to Apostroph.
          $config = new Restclient\Configuration();
          $config->setHost($apostrophsettings['url']);
          $config->setUsername($apostrophsettings['username']);
          $config->setPassword($apostrophsettings['password']);
          $translation_api = new Api\TranslationApi(NULL, $config);
          $translation_data = GeneralHelper::createTranslationRequest($apostrophsettings, $job, $zipPath);
          $translation_request = new Model\TranslationData($translation_data);
          $response = $translation_api->createnewTranslation($translation_request);
          $remote_job_id = $response->getTranslationId();
          $job->set('reference', $remote_job_id);

          $job->submitted("Job sent to provider!");
          $messageTopass .= '<a href="' . file_create_url($zipPath) . '">' . Xss::filter(GeneralHelper::getJobLabelNoSpeChars($job)) . '</a>';
          \Drupal::messenger()->addMessage(Markup::create($messageTopass));
          // As exported files are transfered as zip, delete org. exports as already in the zip.
          // Add zip to transfer array.
          $transferfiles['all'] = GeneralHelper::createFileObject($zipPath);
          foreach ($filearrayexportedfiles as $tempfile) {
            \Drupal::service('file_system')->delete($tempfile->getFileUri());
          }
        }
      } else {
        throw new \Exception("Could not create directory for export: " . $allfilespath);
      }
    } catch (\Exception $exception) {
      // If exception occurs, clean up everything: delete exported files, cancel job in CA if any.
      foreach ($filearrayexportedfiles as $tempfile) {
        if (file_exists($tempfile->getFileUri())) {
          \Drupal::service('file_system')->delete($tempfile->getFileUri());
        }
      }
      if (!file_exists($zipPath) && $ziparchive != NULL) {
        $ziparchive->close();
      }
      $job->setState(Job::STATE_UNPROCESSED);
      try {
        if (isset($createdcpjob)) {
          // TODO: cancel remote job if job created on Apostroph site.
        }
      } catch (\Exception $ex) {
        \Drupal::messenger()->addError("Tried to delete Job on Apostroph platform, but failed: " . $ex->getMessage());
      }
      // TODO: Check why zip cannot be deleted. But after two tests seems to work?
      $zipfileobj = GeneralHelper::createFileObject($zipPath);
      \Drupal::service('file_system')->delete($zipfileobj->getFileUri());
      \Drupal::service('file_system')->deleteRecursive($allfilespath);
      \Drupal::messenger()->addError($exception->getMessage());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function abortTranslation(JobInterface $job)
  {
    // Assume that we can abort a translation job at any time.
    if (!$job->hasTranslator() || $job->getTranslator()->getPluginId() != 'tmgmt_apostrophgroup_provider') {
      return;
    }
    try {
      $translator = $job->getTranslator();
      $apostrophsettings = $translator->getSetting('apostroph-settings');
      $config = new Restclient\Configuration();
      $config->setHost($apostrophsettings['url']);
      $config->setUsername($apostrophsettings['username']);
      $config->setPassword($apostrophsettings['password']);
      $translation_api = new Api\TranslationApi(NULL, $config);
      $response = $translation_api->cancelTranslationWithHttpInfo($job->getReference());
      $job->aborted();
      $job->addMessage(t("Apostroph Job cancelled:") . $job->getReference());
      \Drupal::messenger()->addStatus(t("Apostroph Job cancelled:") . $job->getReference());
      return TRUE;
    } catch (\Exception $exception) {
      \Drupal::logger('TMGMT_APOSTROPH')->error('Error occured while deleting Apostroph job, please contact the responsible project manager: %message', [
        '%message' => $exception->getMessage(),
      ]);
      $job->addMessage(t('Error occured while deleting Apostroph job, please contact the responsible project manager.'), null, 'error');
      $job->aborted();
      return TRUE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultSettings()
  {
    return array(
      'export_format' => 'xlf',
      'scheme' => 'public',
      // Making this setting TRUE by default is more appropriate, however we
      // need to make it FALSE due to backwards compatibility.
      'xliff_processing' => FALSE,
      'xliff_cdata' => FALSE,
      'username' => '',
      'password' => '',
      'one_export_file' => TRUE,
      'customer_id' => '',
      'is_confidential' => FALSE
    );
  }
}
